SystemOrganization addCategory: #StatisticalObjects!

Object subclass: #StatisticsAgent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StatisticalObjects'!

StatisticsAgent subclass: #Accumulator
	instanceVariableNames: 'sum'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StatisticalObjects'!

!Accumulator methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
countDown

	^self input: -1! !

!Accumulator methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
countUp

	^self input: 1! !

!Accumulator methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
input: aNumber

	sum := self sum + aNumber.! !

!Accumulator methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
reset

	sum := nil! !

!Accumulator methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
sum

	sum isNil ifTrue: [ sum := 0 ].
	^sum! !

!Accumulator methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
sum: aNumber

	sum isNil 
		ifTrue: [  self reset. sum := aNumber  ]
		ifFalse: [  self error: 'Setting sum after use is illigal.' ]
! !

Accumulator subclass: #AverageAgent
	instanceVariableNames: 'count mean'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StatisticalObjects'!

!AverageAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
calculateOutput

	self assert: [ self count isZero not ].
	mean := self sum / self count
	! !

!AverageAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
calculateOutputNoAsserting 

	mean := self count isZero
		ifTrue: [0]
		ifFalse: [self sum / self count]
	! !

!AverageAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
count

	count isNil ifTrue: [ count := 0 ].
	^count! !

!AverageAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
input: aNumber

	super input: aNumber.
	count := self count + 1.
	mean := nil! !

!AverageAgent methodsFor: 'as yet unclassified' stamp: 'hjo 2/16/2009 00:44'!
input: aNumber weight: aFloat 

	sum := self sum + (aNumber * aFloat).
	count := self count + aFloat! !

!AverageAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
mean

	mean isNil ifTrue: [ self calculateOutput ].
	^mean! !

!AverageAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
noAssertingMean

	mean isNil ifTrue: [ self calculateOutputNoAsserting ].
	^mean! !

!AverageAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
reset

	super reset.
	count := nil.
! !

AverageAgent subclass: #MeanAndDeviationAgent
	instanceVariableNames: 'squaredSum variance standardDeviation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StatisticalObjects'!

!MeanAndDeviationAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
calculateOutput

	super calculateOutput.
	variance := squaredSum / count - mean squared.
	standardDeviation := variance sqrt! !

!MeanAndDeviationAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
input: aNumber

	super input: aNumber.
	squaredSum := aNumber * aNumber + self squaredSum! !

!MeanAndDeviationAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
reset

	super reset.
	squaredSum := 0! !

!MeanAndDeviationAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
squaredSum

	squaredSum isNil ifTrue: [ squaredSum := 0 ].
	^squaredSum! !

!MeanAndDeviationAgent methodsFor: 'as yet unclassified' stamp: 'hjo 2/14/2009 21:02'!
standardDeviation

	^standardDeviation! !

!MeanAndDeviationAgent methodsFor: 'as yet unclassified' stamp: 'hjo 2/14/2009 12:40'!
standardize: aCollection

	^(aCollection - self mean) / self standardDeviation
	! !

!MeanAndDeviationAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
variance

	^variance! !

Accumulator subclass: #NonnegativeAccumulator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StatisticalObjects'!

!NonnegativeAccumulator methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
input: aNumber

	self assert: [self sum + aNumber >= 0].
	super input: aNumber! !

!StatisticsAgent methodsFor: 'as yet unclassified' stamp: 'hjo 4/13/2008 13:28'!
inputAll: aCollection

	aCollection do: [:object| self input: object ]! !

TestCase subclass: #StatisticAgentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StatisticalObjects'!

!StatisticAgentTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/3/2011 00:46'!
testAccumulator

	| acc |
	acc := Accumulator new.
	self assert: 0 equals: acc sum.
	acc input: 10.
	self assert: 10 equals: acc sum.
	acc input: -20.
	self assert: -10 equals: acc sum.
	acc countUp.
	self assert: -9 equals: acc sum.
	acc countDown.
	self assert: -10 equals: acc sum.
! !

!StatisticAgentTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/3/2011 00:46'!
testAverageAgent

	| av |
	av := AverageAgent new.
	self assert: 0 equals: av sum.
	av input: 10.
	self assert: 10 equals: av mean.
	av input: -20.
	self assert: -5 equals: av mean.
	av countUp.
	self assert: -3 equals: av mean.
	av countDown.
	self assert: -5/2 equals: av mean.
! !

!StatisticAgentTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/3/2011 00:49'!
testAverageAgentNoAssertingMean

	| av |
	av := AverageAgent new.
	self assert: 0 equals: av noAssertingMean.
	self assert: 0 equals: av sum.
	av input: 10.
	self assert: 10 equals: av noAssertingMean.
	av input: -20.
	self assert: -5 equals: av noAssertingMean.
	av countUp.
	self assert: -3 equals: av noAssertingMean.
	av countDown.
	self assert: -5/2 equals: av noAssertingMean.

! !

!StatisticAgentTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/3/2011 00:50'!
testMeanAndDeviationAgent

	| mada |
	mada := MeanAndDeviationAgent new.
	self assert: 0 equals: mada sum.
	mada input: 10.
	self assert: 10 equals: mada sum.
	self assert: 10 equals: mada mean.
	self assert: 0 equals: mada variance.
	self assert: 0 equals: mada standardDeviation.
	mada input: 10.
	self assert: 20 equals: mada sum.
	self assert: 10 equals: mada mean.
	self assert: 0 equals: mada variance.
	self assert: 0 equals: mada standardDeviation.
	mada input: 10.
	self assert: 30 equals: mada sum.
	self assert: 10 equals: mada mean.
	self assert: 0 equals: mada variance.
	self assert: 0 equals: mada standardDeviation.
	mada input: 0.
	self assert: 30 equals: mada sum.
	self assert: 15/2 equals: mada mean.
	self assert: 75/4 equals: mada variance.
	self assert: (mada standardDeviation equals: 4.33012701892219 withRelativeError: 0.0001).
! !

!StatisticAgentTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/3/2011 00:51'!
testNonnegativeAccumulator

	| nacc |
	nacc := NonnegativeAccumulator new.
	self assert: 0 equals: nacc sum.
	nacc input: 100.
	self assert: 100 equals: nacc sum.
	nacc input: -10.
	self assert: 90 equals: nacc sum.
	self should: [nacc input: -100] raise: AssertionFailure.
	nacc input: -10.
	self assert: 80 equals: nacc sum.! !
